/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define REC_TASK_PRIO   ( tskIDLE_PRIORITY + 2 )
#define SEND_TASK_PRIO    ( tskIDLE_PRIORITY + 1 )

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueCAN1;


GPIO_InitTypeDef GPIO_InitStructure;

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);
void CAN_Send(CAN_TypeDef* CANx,CanTxMsg *Message);
void BB_Ext_LED_Init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  CAN1 sends Message and toggles LED orange
  * @param  pvParameters not used
  * @retval None
  */

void vTask_PeriodicSend(void * pvParameters)
{
	uint8_t i;
	CanTxMsg TxMessage;

	TxMessage.StdId = 0x100;
	TxMessage.RTR = CAN_RTR_DATA;
	TxMessage.IDE = CAN_ID_STD;
	TxMessage.DLC = 8;

	for(i=0;i<8;i++)
		TxMessage.Data[i] = 0xff;

	for( ;; )
	{ /* toggle LED each 250ms */
		GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
		GPIO_ToggleBits(GPIOA, GPIO_Pin_8);
		/* 7-Segment a  = PB10  */
		/* 7-Segment b  = PB14  */
		/* 7-Segment c  = PA5   */
		/* 7-Segment d  = PA3   */
		/* 7-Segment e  = PD11  */
		/* 7-Segment f  = PA15  */
		/* 7-Segment g  = PA10  */
		/* 7-Segment dp = PA8   */
		CAN1_Send(&TxMessage);
		vTaskDelay(100);
      }
}
/**
  * @brief  Get recieved Message and toggles LED
  * @param  pvParameters not used
  * @retval None
  */
void vTask_CAN1_MessageRecieved ( void * pvParameters )
{
	CanRxMsg RxMessage;

	xQueueCAN1 = xQueueCreate( 1, sizeof( RxMessage ) );

	if( xQueueCAN1 == 0 )
	{
		while(1)
			;
	}

	for(;;)
	{
		xQueueReceive( xQueueCAN1, &RxMessage, portMAX_DELAY );
		GPIO_ToggleBits(GPIOD, GPIO_Pin_14);
		GPIO_ToggleBits(GPIOC, GPIO_Pin_2);
	}
}

/**
  * @brief  Get recieved Message and toggles LED
  * @param  pvParameters not used
  * @retval None
  */
void vTask_CAN2_MessageRecieved ( void * pvParameters )
{
	uint8_t i;
	uint32_t n = 0;
	CanRxMsg RxMessage;
	CanTxMsg TxMessage;

	TxMessage.StdId = 0x1ff;
	TxMessage.RTR = CAN_RTR_DATA;
	TxMessage.IDE = CAN_ID_STD;
	TxMessage.DLC = 8;


	xQueueCAN2 = xQueueCreate( 1, sizeof( RxMessage ) );

	if( xQueueCAN2 == 0 )
	{
		while(1)
			;
	}

	for(;;)
	{
		xQueueReceive( xQueueCAN2, &( RxMessage ), portMAX_DELAY );
		GPIO_ToggleBits(GPIOD, GPIO_Pin_12);
		GPIO_ToggleBits(GPIOD, GPIO_Pin_6);
		for(i=0;i<8;i++)
			TxMessage.Data[i] = n++;
		CAN2_Send(&TxMessage);
	}
}


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	SystemInit();

  /*!< At this stage the microcontroller clock setting is already configured to
       168 MHz, this is done through SystemInit() function which is called from
       startup file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f4xx.c file
     */

  /*Initialize Leds */
  LED_Init();
  BB_Ext_LED_Init();

  /*Configure the Interrupt*/
  InterruptCAN_Init();

  CAN_Initialisierung();

  /* Create Tasks */
  xTaskCreate( vTask_PeriodicSend, NULL, configMINIMAL_STACK_SIZE, NULL, SEND_TASK_PRIO, NULL);
  xTaskCreate( vTask_CAN1_MessageRecieved ,NULL, configMINIMAL_STACK_SIZE, NULL, SEND_TASK_PRIO, NULL );
  xTaskCreate( vTask_CAN2_MessageRecieved ,NULL, configMINIMAL_STACK_SIZE, NULL, SEND_TASK_PRIO, NULL );

 /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
 /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

  /* Start scheduler */
  vTaskStartScheduler();

  /* We should never get here as control is now taken by the scheduler */
  for( ;; );
}



/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
#ifdef USE_LCD
  /* Initialize the STM324xG-EVAL's LCD */
  STM324xG_LCD_Init();
#endif

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  /* on BB-Ext: */
  /* LED3 = PD6 */ 
  /* LED2 = PC2 */
  /* 7-Segment a  = PB10  */
  /* 7-Segment b  = PB14  */
  /* 7-Segment c  = PA5   */
  /* 7-Segment d  = PA3   */
  /* 7-Segment e  = PD11  */
  /* 7-Segment f  = PA15  */
  /* 7-Segment g  = PA10  */
  /* 7-Segment dp = PA8   */
  /* Initialize Discovery's LEDs */
 
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5|GPIO_Pin_8|GPIO_Pin_10|GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10|GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6|GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);


}

/**
 * @brief  Initializes the LEDs on the extension board
 * @param  None
 * @retval None
 */
void BB_Ext_LED_Init(void)
{


}



/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
